program laplace

   implicit none
   integer, parameter                :: dp=kind(1.d0), n = 100
   integer                           :: maxIter=100000, i, j, iter = 0
   real (dp), dimension(0:n+1,0:n+1) :: T, Tnew
   real (dp)                         :: tol=1.d-4, var = 1.d0, top = 100.d0
   real                              :: starttime, endtime
   integer                           :: ierr

   call cpu_time(startTime)

   T(0:n,0:n) = 0.d0
   T(n+1,1:n) = (/ (i,i=1,n) /) * top / (n+1)
   T(1:n,n+1) = (/ (i,i=1,n) /) * top / (n+1)

   do while (var > tol .and. iter <= maxIter)
      iter = iter + 1
      var = 0.d0
      
      Tnew(1:n,1:n) = 0.25d0 * ( T(0:n-1,1:n) + T(2:n+1,1:n) + T(1:n,0:n-1) + T(1:n, 2:n+1))
      var = maxval(abs( Tnew(1:n,1:n) - T(1:n,1:n) ))

      T(1:n,1:n) = Tnew(1:n,1:n)

      if( mod(iter,100) == 0 ) write(*,"(a,i8,e12.4)") &
         ' iter, variation:', iter, var

   end do

   call cpu_time(endTime)
   write(*,'(/A,F10.4)') ' Elapsed time (s)     =', endTime - startTime
   write(*,*) 'Mesh size            =', n
   write(*,*) 'Stopped at iteration =', iter
   write(*,*) 'The maximum error    =', var
    
   open(10, file='results', action='write', iostat=ierr)
   if(ierr /= 0)  STOP 'Error in opening output file!'
   do j = 1, n
      do i = 1, n
         write(10, "(i8, i8, e18.9)") i,j, T(i,j)
      end do
   end do
   close(10)

end program laplace
